@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 Free Software Foundation, Inc.
+/* Copyright (C) 2001, 2006 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -67,7 +67,7 @@ scm_c_issue_deprecation_warning (const char *msg)
 	  scm_newline (scm_current_error_port ());
 	}
       msg = strdup (msg);
-      iw = scm_malloc (sizeof (struct issued_warning));
+      iw = malloc (sizeof (struct issued_warning));
       if (msg == NULL || iw == NULL)
 	return;
       iw->message = msg;
